(ns com.aurellem.gb.pokedex
  (:use (com.aurellem.gb gb-driver util species))
  (:import [com.aurellem.gb.gb_driver SaveState]))

(def pokedex-seen-start-address 0xD309)

(def pokedex-captured-start-address 0xD2F6)

(defn pokedex-offset [capture? species-num]
  (+ (if capture?
       pokedex-captured-start-address
       pokedex-seen-start-address)
     (int (/  (dec species-num) 8))))
  
(defn read-pokedex
  ([^SaveState state capture? species-num]
     (bit-test
      (aget (memory state)
            (pokedex-offset capture? species-num))
      (rem (dec species-num) 8))))

(defn set-pokedex
  ([^SaveState state capture? set? species-num]
     (let [target (pokedex-offset capture? species-num)
           current-bits
           (aget (memory state)
                 (pokedex-offset capture? species-num))
           bit-offset (rem (dec species-num) 8)]
       (set-memory
        state target
        (if set?
          (bit-set   current-bits bit-offset)
          (bit-clear current-bits bit-offset))))))

(defn read-pokedex-seen
  ([^SaveState state species-or-num]
     (read-pokedex state false
                   (species-name->pokedex-num
                    species-or-num species-or-num)))
  ([species-or-num]
     (read-pokedex-seen @current-state species-or-num)))

(defn set-pokedex-seen
  ([^SaveState state species-or-num seen?]
     (set-pokedex state false seen?
                  (species-name->pokedex-num
                   species-or-num species-or-num)))
  ([species-or-num seen?]
     (set-pokedex-seen @current-state species-or-num seen?)))

(defn read-pokedex-captured
  ([^SaveState state species-or-num]
     (read-pokedex state true
                   (species-name->pokedex-num
                    species-or-num species-or-num)))
  ([species-or-num]
     (read-pokedex-captured @current-state species-or-num)))

(defn set-pokedex-captured
  ([^SaveState state species-or-num captured?]
     (set-pokedex state true captured?
                  (species-name->pokedex-num
                   species-or-num species-or-num)))
  ([species-or-num captured?]
     (set-pokedex-captured @current-state species-or-num captured?)))

(defn print-pokedex
  ([^SaveState state]
     (println "+-----------+-------+-------+")
     (println "|Pokemon    | Seen? | Capt? |")
     (println "+-----------+-------+-------+")
     (dorun
      (map
       #(printf "|%-11s|%s|%s|\n" %1 %2 %3)
       (map #(.toUpperCase (.substring (str %) 1))
            (map pokedex-num->species-name (range 1 152)))
       (map #(if % "   X   " "       ")
            (map read-pokedex-seen (range 1 152)))
       (map #(if % "   X   " "       ")
            (map read-pokedex-captured (range 1 152)))))
     (println "+-----------+-------+-------+"))
  ([] (print-pokedex @current-state)))

(defn give-full-pokedex
  ([^SaveState state]
     (reduce
      (fn [state n]
        (-> state
            (set-pokedex-seen n true)
            (set-pokedex-captured n true)))
      state (range 1 152)))
  ([] (give-full-pokedex @current-state)))

(defn give-empty-pokedex
  ([^SaveState state]
     (reduce
      (fn [state n]
        (-> state
            (set-pokedex-seen n false)
            (set-pokedex-captured n false)))
      state (range 1 152)))
  ([] (give-empty-pokedex @current-state)))
